@@ -21,6 +21,8 @@ pub enum ValType {
     Ip4,
     Sock4,
     Str,
+    Type,
+
     Obj,
     Func,
     Method,
@@ -48,7 +50,6 @@ pub trait Typed {
 /// Represents a static or const version of [Val]. This is used when defining constants in the
 /// stdlib, or when defining default arguments for functions.
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
-#[allow(unused)]
 pub enum ValDef {
     Nil,
     Bool(bool),
@@ -56,6 +57,16 @@ pub enum ValDef {
     Ip4(Ipv4Addr),
     Sock4(SocketAddrV4),
     Str(&'static [u8]),
+    Type(ValType),
+}
+
+impl ValDef {
+    pub fn arg_compatible(&self, v: &Val) -> bool {
+        match self {
+            ValDef::Type(t) => v.is_nil() || v.val_type() == *t,
+            _ => self.val_type() == v.val_type()
+        }
+    }
 }
 
 impl Typed for ValDef {
@@ -68,6 +79,7 @@ impl Typed for ValDef {
             ValDef::Ip4(..) => Ip4,
             ValDef::Sock4(..) => Sock4,
             ValDef::Str(..) => Str,
+            ValDef::Type(..) => Type,
         }
     }
 }
@@ -123,7 +135,6 @@ impl<T> From<&'static T> for ValDef where T: AsRef<[u8]> + ? Sized {
 }
 
 /// Represents a live value in the interpreter, for example the result of evaluating an expression.
-#[allow(unused)]
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Val {
     Nil,
@@ -156,6 +167,7 @@ impl From<ValDef> for Val {
             Ip4(ip) => Val::Ip4(ip),
             Sock4(sock) => Val::Sock4(sock),
             Str(s) => Val::Str(Buf::from(s)),
+            Type(_) => Val::Nil,
         }
     }
 }
@@ -292,6 +304,57 @@ impl From<Val> for Rc<Packet> {
     }
 }
 
+impl From<Val> for Option<u64> {
+    fn from(v: Val) -> Self {
+        match v {
+            Val::Nil => None,
+            Val::U64(u) => Some(u),
+            _ => unreachable!()
+        }
+    }
+}
+
+impl From<Val> for Option<u32> {
+    fn from(v: Val) -> Self {
+        match v {
+            Val::Nil => None,
+            Val::U64(u) => Some(u as u32),
+            _ => unreachable!()
+        }
+    }
+}
+
+impl From<Val> for Option<u16> {
+    fn from(v: Val) -> Self {
+        match v {
+            Val::Nil => None,
+            Val::U64(u) => Some(u as u16),
+            _ => unreachable!()
+        }
+    }
+}
+
+impl From<Val> for Option<u8> {
+    fn from(v: Val) -> Self {
+        match v {
+            Val::Nil => None,
+            Val::U64(u) => Some(u as u8),
+            _ => unreachable!()
+        }
+    }
+}
+
+impl From<Val> for Option<Buf> {
+    fn from(v: Val) -> Self {
+        match v {
+            Val::Nil => None,
+            Val::Str(s) => Some(s),
+            _ => unreachable!()
+        }
+    }
+}
+
+
 impl<T: 'static + Obj> From<T> for Val {
     fn from(obj: T) -> Self {
         Val::Obj(ObjRef::from(obj))
@@ -368,7 +431,6 @@ impl Val {
         }
     }
 
-    #[allow(dead_code)]
     pub fn lookup_symbol(&self, name: &str) -> Result<Symbol, Error> {
 		match self {
 			Val::Obj(obj) => obj.lookup_symbol(name).ok_or(NameError),
